// Copyright(C)SEGA 
#include "Collada/Node.h"
#include "Collada/Extra.h"
#include "Collada/Util.h"
#include "Collada/Collection.h"
#include "Collada/VisualScene.h"
#include "Collada/Macro.h"
#include "Collada/InstanceGeometry.h"
#include "Collada/XML.h"

#include <sstream>

using namespace std;

namespace Collada {

    // RXgN^
    Node::Node() : mNodeType(NodeType_NODE),
        mInstanceType(InstanceType_NONE),
        mInstanceGeometry(NULL),
        mInstanceLight(NULL),
        mInstanceCamera(NULL),	
        mExtra(NULL) ,
        mTranslate( 0.0f, 0.0f, 0.0f ),
        mRotationX( 0.0f, 0.0f, 0.0f, 0.0f ),
        mRotationY( 0.0f, 0.0f, 0.0f, 0.0f ),
        mRotationZ( 0.0f, 0.0f, 0.0f, 0.0f ),
        mScale( 1.0f, 1.0f, 1.0f )
    {
    }

    // fXgN^
    Node::~Node()
    {
        SAFE_DELETE( mExtra );
        SAFE_DELETE( mInstanceGeometry );
        SAFE_DELETE( mInstanceLight );
        SAFE_DELETE( mInstanceCamera );		

        const int childNumber = mChildren.size();
        for( int i=0; i<childNumber; i++ ) {

            SAFE_DELETE( mChildren[ i ] );

        }        
    }

    // NodeType𕶎ŕԂ
    const char* Node::nodeTypeStr( const Node::NodeType nodeType )
    {
        switch( nodeType ) {

        case NodeType_JOINT:
            return "JOINT";

        case NodeType_NODE: 
            return "NODE";
        }

        return "";
    }

    // XMLݒ
    void Node::toXML( XMLElement* e )
    {
        e->mName = "node";        
        e->mAttributes.push_back( new XMLAttribute( "id", mId ) );
        e->mAttributes.push_back( new XMLAttribute( "name", mName ) );

        if( mNodeType == NodeType_JOINT ) {

            e->mAttributes.push_back( new XMLAttribute( "type", nodeTypeStr( mNodeType ) ) );

        }
        {            
            {
                XMLElement* translateE = new XMLElement();

                ostringstream ss;
                ss << mTranslate.x << " " << mTranslate.y << " " << mTranslate.z;

                translateE->mName = "translate";                
                translateE->mAttributes.push_back( new XMLAttribute( "sid", "translate" ) );                
                translateE->mValue = ss.str();

                e->mChildren.push_back( translateE );
            }			


            {
                XMLElement* rotateXE = new XMLElement();

                ostringstream ss;
                ss << mRotationX.x << " " << mRotationX.y << " " << mRotationX.z << " " << mRotationX.w;

                rotateXE->mName = "rotate";                
                rotateXE->mAttributes.push_back( new XMLAttribute( "sid", "rotateX" ) );
                rotateXE->mValue = ss.str();

                e->mChildren.push_back( rotateXE );
            }

            {
                XMLElement* rotateYE = new XMLElement();

                ostringstream ss;
                ss << mRotationY.x << " " << mRotationY.y << " " << mRotationY.z << " " << mRotationY.w;

                rotateYE->mName = "rotate";                
                rotateYE->mAttributes.push_back( new XMLAttribute( "sid", "rotateY" ) );
                rotateYE->mValue = ss.str();

                e->mChildren.push_back( rotateYE );
            }

            {
                XMLElement* rotateZE = new XMLElement();

                ostringstream ss;
                ss << mRotationZ.x << " " << mRotationZ.y << " " << mRotationZ.z << " " << mRotationZ.w;

                rotateZE->mName = "rotate";                
                rotateZE->mAttributes.push_back( new XMLAttribute( "sid", "rotateZ" ) );
                rotateZE->mValue = ss.str();

                e->mChildren.push_back( rotateZE );
            }

            // scale            
            {
                XMLElement* scaleE = new XMLElement();

                ostringstream ss;
                ss << mScale.x << " " << mScale.y << " " << mScale.z;

                scaleE->mName = "scale";                
                scaleE->mAttributes.push_back( new XMLAttribute( "sid", "scale" ) );
                scaleE->mValue = ss.str();

                e->mChildren.push_back( scaleE );
            }

            // QƐ̎ނƂ
            switch( mInstanceType ) {

            case InstanceType_NONE:
                break;

            case InstanceType_LIGHT:
                {
                    XMLElement* instanceLightE = new XMLElement();
                    instanceLightE->mName = "instance_light";                    
                    instanceLightE->mAttributes.push_back( new XMLAttribute( "url", mInstanceLight->mURL ) );
                    e->mChildren.push_back( instanceLightE );
                }
                break;			

            case InstanceType_CAMERA:			
                {
                    XMLElement* instanceCameraE = new XMLElement();
                    instanceCameraE->mName = "instance_camera";
                    instanceCameraE->mAttributes.push_back( new XMLAttribute( "url", mInstanceCamera->mURL ) );
                    e->mChildren.push_back( instanceCameraE );
                }
                break;			

            case InstanceType_GEOMETRY:
                {
                    XMLElement* instanceGeometryE = new XMLElement();
                    instanceGeometryE->mName = "instance_geometry";
                    instanceGeometryE->mAttributes.push_back( new XMLAttribute( "url", mInstanceGeometry->mURL ) );                    
                    {
                        XMLElement* bindMaterialE = new XMLElement();
                        bindMaterialE->mName = "bind_material";                        
                        {
                            XMLElement* techniqueCommonE = new XMLElement();
                            techniqueCommonE->mName = "technique_common";                            
                            {
                                const int instanceMaterialNumber = mInstanceGeometry->mInstanceMaterials.size();
                                for( int m=0; m<instanceMaterialNumber; m++ ) {

                                    XMLElement* instanceMaterialE = new XMLElement();
                                    instanceMaterialE->mName = "instance_material";
                                    instanceMaterialE->mAttributes.push_back( new XMLAttribute( "symbol", mInstanceGeometry->mInstanceMaterials[ m ].mSymbol ) );
                                    instanceMaterialE->mAttributes.push_back( new XMLAttribute( "target", mInstanceGeometry->mInstanceMaterials[ m ].mTarget ) );
                                    techniqueCommonE->mChildren.push_back( instanceMaterialE );

                                }
                            }
                            bindMaterialE->mChildren.push_back( techniqueCommonE );

                        }
                        instanceGeometryE->mChildren.push_back( bindMaterialE );

                    }
                    e->mChildren.push_back( instanceGeometryE );
                }
                break;

            }

        }

        // q
        const int childNumber = mChildren.size();
        for( int child=0; child<childNumber; child++ ) {

            // 蔲VisualScene͂Pőv낤
            Node* childNode = mChildren[ child ];
            if( childNode ) {

                XMLElement* childE = new XMLElement();
                childNode->toXML( childE );
                e->mChildren.push_back( childE );

            }

        }

        // Extra
        if( mExtra ) {

            XMLElement* extraE = new XMLElement();
            mExtra->toXML( extraE );
            e->mChildren.push_back( extraE );

        }

    }

}
